<?php

namespace UBRIR;

abstract class CMS_PaymentCallback
{
    protected $gatewayPanel;
    protected $callbackDataContainer;
    protected $paymentStatus;
    /**
     * @var PaymentMailNotification
     */
    protected $paymentMailNotification;
    protected $receiverEmails;

    protected $shopOrderId;
    protected $shopPaymentId;
    protected $continueLink;
    protected $resultData;

    function __construct(InterfaceConnectionConfig $connector = null)
    {
        $this->gatewayPanel = new GatewayInteractionPanel($connector);

        $this->callbackDataContainer = $this->HandleCallback();
    }

    public function GetShopOrderId()
    {
        if ($this->shopOrderId == null)
            throw new \Exception('Не выполнена обработка запроса методом ProcessCallbackResponse');

        return $this->shopOrderId;
    }

    public function GetPaymentId()
    {
        if ($this->shopPaymentId == null)
            throw new \Exception('Не выполнена обработка запроса методом ProcessCallbackResponse');

        return $this->shopPaymentId;
    }

    private function SetPaymentState($orderState)
    {
        if (TypeEnums::isValidOrderState($orderState))
            $this->paymentStatus = $orderState;
        else
            $this->paymentStatus = null;
    }

    public function isPaymentApproved()
    {
        if ($this->paymentStatus == TypeEnums::OrderStatus_APPROVED)
            return true;

        return false;
    }

    public function GetPaymentStatus()
    {
        return $this->paymentStatus;
    }

    public function GetPaymentDescription()
    {
        return $this->callbackDataContainer->getFieldValue('OrderStatusScr');
    }

    public function GetOrderSum()
    {
        $floatSum = (float)$this->callbackDataContainer->getFieldValue('PurchaseAmount') / 100;

        return $floatSum;
    }

    //region Почтовые уведомления
    public function ActivatePaymentMailNotification(PaymentMailNotification $paymentMailNotification)
    {
        $this->paymentMailNotification = $paymentMailNotification;

        return $this;
    }

    /**
     * Можно указывать несколько получателей
     * $receiverEmail = Array('Тест Тестовов' => 'test@test.com');
     *
     * @param $receiverEmail
     * @param string $receiverName
     * @return mixed
     */
    protected function SetMailReceivers($receiverEmail, $receiverName = '')
    {
        $this->receiverEmails = $receiverEmail;

        return $this;
    }

    //endregion


    private function HandleCallback()
    {
        $this->gatewayPanel->Log('Обработка callback от шлюза');

        if (!isset($_POST['xmlmsg'])) {
            $response = new DataContainer();
            $response->AddError('Неверный запрос сервера');
        }

        $XMLString = base64_decode($_POST['xmlmsg']);
        $xmlLines = explode("\n", $XMLString);

        //Проверяем что с XML все в порядке (по большому счету это нужно только как защита от прямых дерганий callback
        libxml_use_internal_errors(); //Подавляем вывод ошибок XML
        simplexml_load_string($XMLString);
        $xmlErrors = libxml_get_errors();
        if (count($xmlErrors) > 0) {
            $this->gatewayPanel->Log('Обнаружены ошибки в XML');
            foreach ($xmlErrors as $error) {
                $this->gatewayPanel->Log(self::FormatXMLError($error, $xmlLines));
            }

            $response = new DataContainer();
            $response->AddError('Обнаружены ошибки в XML');
            $this->gatewayPanel->LogErrors($response);
        } else {
            $this->gatewayPanel->Log('Callback: Разбирем полученный XML');
            $this->gatewayPanel->Log($XMLString);

            $objXML = simplexml_load_string($XMLString);
            $arKeyValueXML = ServerConnector::XML2Array($XMLString);
            $response = new DataContainer();
            $response->setSuccess()->SaveXMLData($XMLString, $objXML, $arKeyValueXML);
        }

        return $response;
    }

    public function isRequestSuccessfullyParsed()
    {
        return $this->callbackDataContainer->isSuccess();
    }

    public function ProcessCallbackResponse()
    {
        if ($this->callbackDataContainer->isSuccess()) {
            $orderStatus = $this->callbackDataContainer->getFieldValue('OrderStatus');
            $TWPG_OrderID = $this->callbackDataContainer->getFieldValue('OrderId');

            $dbRow = $this->gatewayPanel->GetOrderDataByUBRIROrderID($TWPG_OrderID);
            if ($dbRow === false) {
                //TODO: только обработка, выводом шаблонов и ветвлением по ним должен заняться ShowCallbackProcessResult
                $this->gatewayPanel->ShowPaymentError(false, 'Не найден заказ с платежным идентификатором = ' . $TWPG_OrderID);
                return;
            }

            $this->shopOrderId = intval($dbRow['order_id']);
            $this->shopPaymentId = intval($dbRow['payment_id']);
            $SessionId = $dbRow['session_id'];

            $this->resultData = new PaymentResultViewer();
            $this->resultData
                ->SetPaymentTime($this->callbackDataContainer->getFieldValue('date'))
                ->SetShopOrderId($this->shopOrderId)
                ->SetTWPG_OrderId($TWPG_OrderID)
                ->SetCardPAN($this->callbackDataContainer->getFieldValue('PAN'))
                ->SetCardOwner($this->callbackDataContainer->getFieldValue('Name'))
                ->SetPaymentAmount($this->callbackDataContainer->getFieldValue('PurchaseAmountScr'));

            $this->SetPaymentState($orderStatus);

            switch ($orderStatus) {
                case TypeEnums::OrderStatus_APPROVED: {

                    $checkOrderStatusResponse = $this->gatewayPanel->SetupAndCheckOrderStatus($TWPG_OrderID, $SessionId);

                    if ($this->paymentMailNotification !== null) {
                        $resultData = $this->BeforeSendMailNotification($this->resultData);

                        $this->paymentMailNotification->ConnectResultViewer($resultData);

                        if (!empty($this->receiverEmails)) {
                            $this->gatewayPanel->Log('Отправляем сообщение получателям:');
                            $this->gatewayPanel->Log($this->receiverEmails);
                            $this->paymentMailNotification->SendPaymentNotification($this->receiverEmails);
                        }
                    }

                    if (!$checkOrderStatusResponse->isSuccess()) {
                        $bankResponseDescription = $checkOrderStatusResponse->getFieldValue('ResponseDescription');

                        $this->resultData->AddExtraMessage($bankResponseDescription, 'Сообщение');
                    } else {
                        $this->continueLink = $this->OnPaymentApproved($this->shopOrderId, $SessionId);
                    }
                }
                    break;
                case TypeEnums::OrderStatus_CANCELED: {
                    $this->resultData->SetMessageBaseOnResponse($this->callbackDataContainer);
                    $this->continueLink = $this->OnPaymentCanceled($this->shopOrderId, $SessionId);
                }
                    break;
                case TypeEnums::OrderStatus_DECLINED: {
                    $this->resultData->SetMessageBaseOnResponse($this->callbackDataContainer);

                    $this->continueLink = $this->OnPaymentDeclined($this->shopOrderId, $SessionId);
                }
                    break;
                default:
                    break;
            }
        }

        return $this;
    }

    public function ShowCallbackProcessResult()
    {
        if ($this->paymentStatus == null)
            $this->ProcessCallbackResponse();

        if (!is_string($this->continueLink)) {
            $this->continueLink = $this->gatewayPanel->GetCMSConnector()->GetDefaultContinueLink();
        }

        if ($this->callbackDataContainer->isSuccess()) {

            switch ($this->paymentStatus) {
                case TypeEnums::OrderStatus_APPROVED: {
                    $successMessage = self::GetPaymentDescription();

                    $this->gatewayPanel->ShowPaymentSuccess($this->continueLink, $successMessage, $this->resultData);

                    return true;
                }
                    break;

                case TypeEnums::OrderStatus_CANCELED: {
                    $this->gatewayPanel->ShowPaymentError($this->continueLink, '', $this->resultData);

                    return false;
                }
                    break;

                case TypeEnums::OrderStatus_DECLINED: {
                    $this->gatewayPanel->ShowPaymentError($this->continueLink, '', $this->resultData);

                    return false;
                }
                    break;
            }

            $this->gatewayPanel->ShowPaymentError(false, '', $this->resultData);
        } else {
            $this->gatewayPanel->ShowPaymentError(false, 'Неверный формат ответа');
        }

        return false;
    }

    private function FormatXMLError($error, $xml)
    {
        $return = $xml[$error->line - 1] . "\n";
        $return .= str_repeat('-', $error->column) . "^\n";

        switch ($error->level) {
            case LIBXML_ERR_WARNING:
                $return .= "Warning $error->code: ";
                break;
            case LIBXML_ERR_ERROR:
                $return .= "Error $error->code: ";
                break;
            case LIBXML_ERR_FATAL:
                $return .= "Fatal Error $error->code: ";
                break;
        }

        $return .= trim($error->message) .
            "\n  Line: $error->line" .
            "\n  Column: $error->column";

        if ($error->file) {
            $return .= "\n  File: $error->file";
        }

        return "$return\n\n--------------------------------------------\n\n";
    }

    /**
     * @param PaymentResultViewer $resultData
     * @return PaymentResultViewer
     */
    abstract public function BeforeSendMailNotification(PaymentResultViewer $resultData);

    /**
     * @param $shopOrderId
     * @param $sessionId
     * @return string
     */
    abstract public function OnPaymentApproved($shopOrderId, $sessionId);

    abstract public function OnPaymentCanceled($shopOrderId, $sessionId);

    abstract public function OnPaymentDeclined($shopOrderId, $sessionId);
}